
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/logging/log.h>
#include <zephyr/kernel.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/crc.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "device_cfg.h"										// basic configuration
#include "driver_ESB.h"  									/* constants, macros and function for ESB service */

// *****************************************************************************
LOG_MODULE_REGISTER(esb_prx, CONFIG_ESB_PRX_APP_LOG_LEVEL);
static ESBcontrol_t * Esb_driver_ptr = NULL;                // pointer to ESB instance

static K_SEM_DEFINE(esb_tx_done_sem, 0, 1);                 // A semaphore used to wait for each TX to complete.
static K_SEM_DEFINE(esb_rx_done_sem, 0, 1);                 // A semaphore used to wait for each RX to complete.
static K_FIFO_DEFINE(esb_rx_data_fifo);						// used for ESB retransmit from ESB handler
// *****************************************************************************
/* Event handler for ESB (interrupt context). */
void drv_esb_event_handler(struct esb_evt const *event_ptr)
{
//    static struct esb_payload rx_payload;
	static struct data_esb_item_t data_esb_rx_item;//[ESB_RX_QUEUE_ITEMS_NUM];

    if(Esb_driver_ptr) {
		switch (event_ptr->evt_id) {
			case ESB_EVENT_TX_SUCCESS:
	//			LOG_DBG("TX SUCCESS EVENT");
				dk_set_led_on(1);
				dk_set_led_off(0);
				k_sem_give(&esb_tx_done_sem);					// it is released when started
				Esb_driver_ptr->tx_ready = true;
				break;

		case ESB_EVENT_TX_FAILED:
				dk_set_led_off(0);
				LOG_DBG("TX FAILED EVENT");
				k_sem_give(&esb_tx_done_sem);
				Esb_driver_ptr->tx_ready = true;
				break;

		case ESB_EVENT_RX_RECEIVED:
	#if TEST_DRV_ESB_EVENT_RX_PRINT
				while (esb_read_rx_payload(&rx_payload) == 0) {
					LOG_DBG("Packet received, len %03d : "
						"0x%02x, 0x%02x, 0x%02x, 0x%02x", rx_payload.length,
						rx_payload.data[0],	rx_payload.data[1],
						rx_payload.data[2], rx_payload.data[3] );
				}
	#endif
				dk_set_led_on(2);
				k_sem_give(&esb_rx_done_sem);
	// 			data_esb_rx_item.read_status = esb_read_rx_payload(&data_esb_rx_item.payload);		// reads the received ESB packet
	// #if DRV_ESB_RX_DATA_FIFO_PUT && (TRANSMIT_ESB_MODE==ESB_MODE_PRX)
	// 			k_fifo_put(&esb_rx_data_fifo, &data_esb_rx_item);
	// #endif
				break;
		}
    }
}

// *****************************************************************************
int drv_esb_init(ESBcontrol_t * esb_ptr, ESBconfig_t * esb_cfg)  // Attention, a function named "esb_init()" already exists in esb.c.
{
	int err;
	/* These are arbitrary default addresses. In end user products
	 * different addresses should be used for each set of devices.
	 */
    Esb_driver_ptr = esb_ptr;                               // catch pointer to instance
    esb_ptr->tx_sem_ptr = &esb_tx_done_sem;
	esb_ptr->rx_sem_ptr = &esb_rx_done_sem;
	esb_ptr->rx_fifo_ptr = &esb_rx_data_fifo;
    esb_ptr->tx_ready = true;
    esb_ptr->ack_enable = esb_cfg->ack_enable;

	struct esb_config config = ESB_DEFAULT_CONFIG;

	config.protocol = ESB_PROTOCOL_ESB_DPL;
	config.retransmit_delay = 250;
	config.bitrate = ESB_BITRATE_4MBPS;
	config.mode = esb_cfg->mode;
	config.event_handler = drv_esb_event_handler;
	config.payload_length = CONFIG_ESB_MAX_PAYLOAD_LENGTH;
//    config.selective_auto_ack = esb_cfg->ack_enable ? true : false;   // identity - old try
    config.selective_auto_ack = esb_cfg->ack_enable ? false : true;   // inversion of user config: Selective auto acknowledgement.
				   // When this feature is disabled, all packets will be acknowledged ignoring the noack field.

	if (IS_ENABLED(CONFIG_ESB_FAST_SWITCHING)) {
		config.use_fast_ramp_up = true;
	}
		
#if TEST_DRV_ESB_INIT_PARAM_PRINT
    LOG_DBG("base_addr_0_ptr=%p, base_addr_1_ptr=%p, addr_prefix_ptr=%p ",
        esb_cfg->base_addr_0_ptr, esb_cfg->base_addr_1_ptr, esb_cfg->addr_prefix_ptr);
	LOG_DBG("tx_sem_ptr=%p, rx_fifo_ptr=%p, ack_enable=%d, selective_auto_ack=%d, use_fast_ramp_up=%d", 
		esb_ptr->tx_sem_ptr, esb_ptr->rx_fifo_ptr, esb_ptr->ack_enable, config.selective_auto_ack, config.use_fast_ramp_up);
#endif

	err = esb_init(&config);
	if (err) {
        LOG_ERR("esb_init() failed, err %d", err);
		return err;
	}

	err = esb_set_base_address_0(esb_cfg->base_addr_0_ptr);
	if (err) {
        LOG_ERR("esb_set_base_address_0() failed, err %d", err);
		return err;
	}

	err = esb_set_base_address_1(esb_cfg->base_addr_1_ptr);
	if (err) {
        LOG_ERR("esb_set_base_address_1() failed, err %d", err);
		return err;
	}

	err = esb_set_prefixes(esb_cfg->addr_prefix_ptr, ESB_PREFIX_SIZE);
	if (err) {
        LOG_ERR("esb_set_prefixes() failed, err %d", err);
		return err;
	}

	return 0;
}

// *****************************************************************************

// *****************************************************************************

